-----------------------------------------------------------------------------------------------
-- Client Lua Script for RosterBoss
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- RosterBoss Module Definition
-----------------------------------------------------------------------------------------------
local RosterBoss = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function RosterBoss:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
    return o
end

function RosterBoss:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 
-----------------------------------------------------------------------------------------------
-- RosterBoss OnLoad
-----------------------------------------------------------------------------------------------
function RosterBoss:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("RosterBoss.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- RosterBoss OnDocLoaded
-----------------------------------------------------------------------------------------------
function RosterBoss:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "RosterBossForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end	
		Apollo.RegisterSlashCommand("rb", "OnRosterBossToggle", self)
		Apollo.RegisterEventHandler("GuildRoster", "OnGetGuildRoster", self)
		Apollo.RegisterEventHandler("Group_Join", "OnGroupFormed", self)
		Apollo.RegisterTimerHandler("intervalTimer", "Invite", self)
		Apollo.RegisterTimerHandler("durationTimer", "StopAutoInvites", self)
		local wndMainLocation = WindowLocation.new(self.savedLocation)
		if wndMainLocation then 
			if self.savedLocation and self.savedLocation.nOffsets[1] ~= 0 then
				self.wndMain:MoveToLocation(wndMainLocation)
				self.savedLocation = nil 
			end
    	end
		--local rnd = math.random(4)

		self.guildRanks = nil
		self.durationTimer = nil
		self.intervalTimer = nil
		self.timeMulti = 60
		self.startButton = self.wndMain:FindChild("StartInvites")
		self.stopButton = self.wndMain:FindChild("StopInvites")
		self.stopButton:Show(false)
		self.intervalSlider = self.wndMain:FindChild("IntervalSlider")
		self.intervalValue = self.wndMain:FindChild("IntervalValue")
		self.durationSlider = self.wndMain:FindChild("DurationSlider")
		self.durationValue = self.wndMain:FindChild("DurationValue")
		self.convertToRaidCB = self.wndMain:FindChild("ConvertToRaid")
		self.openInvitesCB = self.wndMain:FindChild("OpenInvites")
		if self.rankChoices == nil then
			self.rankChoices = {}
		end
		self.checkBoxes = {}
		for x=1, 10 do
			self.checkBoxes[x] = self.wndMain:FindChild("CheckBox"..x)
			self.checkBoxes[x]:SetCheck(self.rankChoices[x])
		end
		self.convertToRaidCB:SetCheck(self.convertToRaid)
		self.openInvitesCB :SetCheck(self.openInvites)
		self:BuildRankBoxes()
		if self.interval ~= nil then
			self.intervalSlider:SetValue(self.interval)
			self.intervalValue:SetText(self.interval)
		end
		if self.duration ~= nil then
			self.durationSlider:SetValue(self.duration)
			self.durationValue:SetText(self.duration)
		end
		self.GetGuildRoster()
		self.wndMain:Show(self.visibility)
		self.inviteQueue = {last = 0}
		self.popTimer = nil
		self:InvitePop()
	end
end

-----------------------------------------------------------------------------------------------
-- RosterBoss Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/rb"
function RosterBoss:OnRosterBossToggle()
	self.visibility = not self.visibility
	self.wndMain:Show(self.visibility)
end

function RosterBoss:GetRankChoices()
	for x=1, #self.checkBoxes do
		if self.checkBoxes[x]:IsStyleOn("IgnoreMouse") then self.checkBoxes[x]:SetCheck(false) end
		self.rankChoices[x] = self.checkBoxes[x]:IsChecked()
	end
	self.convertToRaid = self.convertToRaidCB:IsChecked()
	self.openInvites = self.openInvitesCB:IsChecked()
end

function RosterBoss:BuildRankTable()
	--for k, v in pairs(
	return
end

function RosterBoss:GetGuild()
	for _,guild in ipairs(GuildLib.GetGuilds()) do
		if guild:GetType() == GuildLib.GuildType_Guild then
			return guild
		end
	end
	return nil
end

function RosterBoss:BuildRankBoxes()
	local checkBoxCount = 1
	local rankCount = 1
	for _,rank in ipairs(self:GetGuild():GetRanks()) do
		if rank.strName ~= "" then
			self.checkBoxes[rankCount]:SetText(rank.strName)
			self.checkBoxes[rankCount]:SetOpacity(1)
			self.checkBoxes[rankCount]:SetTextColor("FF00FFFF")
			self.checkBoxes[rankCount]:RemoveStyle("IgnoreMouse")	
		else
			self.checkBoxes[rankCount]:SetText("---")
			self.checkBoxes[rankCount]:SetCheck(false)
			self.checkBoxes[rankCount]:SetOpacity(0.5)
			self.checkBoxes[rankCount]:SetTextColor("5500FFFF")
			self.checkBoxes[rankCount]:AddStyle("IgnoreMouse")	
		end
		rankCount = rankCount + 1
	end
end

function RosterBoss:GetGuildRoster()
	for _,guild in ipairs(GuildLib.GetGuilds()) do
        if guild:GetType() == GuildLib.GuildType_Guild then
			guild:RequestMembers()
		end
	end
end

function RosterBoss:OnGetGuildRoster(guildCurr, tRoster)
	self.roster = tRoster
end
-----------------------------------------------------------------------------------------------
-- RosterBossForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function RosterBoss:OnOK()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function RosterBoss:OnCancel()
	self.wndMain:Close() -- hide the window
end

function RosterBoss:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	local tsave = {}
	tsave.version = 1.0
	tsave.savedLocation = self.wndMain:GetLocation():ToTable()
	tsave.visibility = self.visibility
	tsave.interval = self.interval
	tsave.duration = self.duration
	self:GetRankChoices()
	tsave.rankChoices = self.rankChoices 
	tsave.convertToRaid = self.convertToRaid
	tsave.openInvites = self.openInvites
	return tsave
end

function RosterBoss:OnRestore(eLevel, saveData)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	self.version = saveData.version
	if self.version ~= 1.0 then
		self.savedLocation = nil
		self.visibility = true
		self.interval = 3
		self.duration = 30
		self.rankChoices = {}
		self.convertToRaid = false
		self.openInvites = false 
	else
		self.savedLocation = saveData.savedLocation
		self.visibility = saveData.visibility
		self.interval = saveData.interval 
		self.duration = saveData.duration 
		self.rankChoices = saveData.rankChoices 
		self.convertToRaid = saveData.convertToRaid
		self.openInvites = saveData.openInvites 
	end
end

function RosterBoss:OnIntervalChanged( wndHandler, wndControl, fNewValue, fOldValue )
	self.intervalValue:SetText(fNewValue)
	self.interval = fNewValue
end

function RosterBoss:OnDurationChanged( wndHandler, wndControl, fNewValue, fOldValue )
	self.durationValue:SetText(fNewValue)
	self.duration = fNewValue
end

function RosterBoss:StartAutoInvites( wndHandler, wndControl, eMouseButton )
	self.startButton:Show(false)
	self.stopButton:Show(true)
	if self.durationTimer ~= nil then
		Print("Invite timer is already running")
	else
		--self.duration = nil
		if self.duration == nil then
			self.duration = 30 
			self:OnDurationChanged(nil,nil,30,nil)	
		end
		self.durationTimer = ApolloTimer.Create(self.duration * self.timeMulti, false, "StopAutoInvites", self)
		self:AutoInvite()
	end
end

function RosterBoss:AutoInvite()
	self:Invite()
	--self.interval = nil
	if self.interval == nil then
		self.interval = 3 
		self:OnIntervalChanged(nil,nil,3,nil)	
	end
	self.intervalTimer =  ApolloTimer.Create(self.interval * self.timeMulti, false, "AutoInvite", self)
end

function RosterBoss:StopAutoInvites( wndHandler, wndControl, eMouseButton )
	if self.intervalTimer ~= nil then
		self.intervalTimer:Stop()
		self.intervalTimer = nil
	end
	if self.durationTimer ~= nil then
		self.durationTimer:Stop()
		self.durationTimer = nil
	end
	self.stopButton:Show(false)
	self.startButton:Show(true)
end

function RosterBoss:OnGroupFormed()
	if GroupLib.AmILeader() then
		if self.openInvites and GroupLib.GetJoinRequestMethod ~= 0 then
			GroupLib.SetJoinRequestMethod(0)
		end
		if self.convertToRaid and not GroupLib.InRaid() then
			GroupLib.ConvertToRaid()
		end
	end
end

function RosterBoss:InvitePush(name)
	local last = self.inviteQueue.last + 1
	self.inviteQueue.last = last
	self.inviteQueue[last] = name
end

function RosterBoss:InvitePop()
	local last = self.inviteQueue.last
	if last > 0 then -- queue is not empty
		local name = self.inviteQueue[last]
		self.inviteQueue[last] = nil
		self.inviteQueue.last = last - 1
		--Print("Popped " .. name)
		GroupLib.Invite(name, GameLib.GetRealmName(), "RosterBoss!")
	else
		--Print("Queue empty")
	end
	self.popTimer =  ApolloTimer.Create(1, false, "InvitePop", self)

end

function RosterBoss:Invite( wndHandler, wndControl, eMouseButton )
	if self.roster == nil then
		Print("No roster.")
	else
		for rank, selected in pairs(self.rankChoices) do
			if selected then
				Print("Inviting rank "..rank.." members")
				for _, member in pairs(self.roster) do
					if member.nRank == rank and member.fLastOnline == 0 then 
						local inGroup = false
						if member.strName == GameLib.GetPlayerUnit():GetName() then inGroup = true end
						for m=1, GroupLib.GetMemberCount() do
							if member.strName == GroupLib.GetGroupMember(m).strCharacterName then
								inGroup = true
							end
						end
						if not inGroup then
							self:InvitePush(member.strName)
						end
					end
				end	
			end
		end
	end		
end

-----------------------------------------------------------------------------------------------
-- RosterBoss Instance
-----------------------------------------------------------------------------------------------
local RosterBossInst = RosterBoss:new()
RosterBossInst:Init()
